// --------------------------------------------------------------------------------------------------
// PARAMETERS:
// --------------------------------------------------------------------------------------------------

//transforms
float4x4 tW: WORLD;        //the models world matrix
float4x4 tV: VIEW;         //view matrix as set via Renderer (EX9)
float4x4 tP: PROJECTION;
float4x4 tWVP: WORLDVIEWPROJECTION;

//texture
texture Src <string uiname="Source";>;
sampler SrcSamp = sampler_state    //sampler for doing the texture-lookup
{
    Texture   = (Src);          //apply a texture to the sampler
    MipFilter = LINEAR;         //sampler states
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};


//source transformation
float4x4 tTex: TEXTUREMATRIX <string uiname="Source Transform";>;


//pin for the Pixelshift
float PixelX
<
    string uiname="PixelX";
    float uimin=0.0f;
    float uimax=1.0f;
> = 0.2f;

float PixelY
<
    string uiname="PixelY";
    float uimin=0.0f;
    float uimax=1.0f;
> = 0.2f;


//the data structure: "vertexshader to pixelshader"
//used as output data with the VS function
//and as input data with the PS function
struct vs2ps
{
    float4 Pos  : POSITION;
    float2 TexCd : TEXCOORD0;
    float2 Pixelsize : TEXCOORD1;
 };

// --------------------------------------------------------------------------------------------------
// VERTEXSHADERS
// --------------------------------------------------------------------------------------------------
vs2ps VS(
    float4 Pos  : POSITION,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;

    //transform position
    Out.Pos = mul(Pos, tWVP);
    
    //transform texturecoordinates
    Out.TexCd = mul(TexCd, tTex);

    return Out;
}

// --------------------------------------------------------------------------------------------------
// PIXELSHADERS:
// --------------------------------------------------------------------------------------------------


float4 ps_Pixel(vs2ps In): COLOR
{
    //source texture lookup
    float4 outColor = tex2D(SrcSamp, float2(PixelX,PixelY));

    return outColor;
}


float4 ps_ShowPixel(vs2ps In): COLOR
{
    //source texture lookup
    float4 outColor = tex2D(SrcSamp, In.TexCd);

    if (In.TexCd.x < PixelX+0.01f && In.TexCd.x > PixelX-0.01f) {
     if (In.TexCd.y < PixelY+0.01f && In.TexCd.y > PixelY-0.01f) {
        outColor = float4 (1.0f,0.0f,0.0f,1.0f);
     }
    }

    return outColor;
}

// --------------------------------------------------------------------------------------------------
// TECHNIQUES:
// --------------------------------------------------------------------------------------------------


technique Pixel
{
    pass P0
    {
        VertexShader = compile vs_1_1 VS();
        PixelShader  = compile ps_2_0 ps_Pixel();
    }
}

technique ShowPixel
{
    pass P0
    {
        VertexShader = compile vs_1_1 VS();
        PixelShader  = compile ps_2_0 ps_ShowPixel();
    }
}

technique TFixedFunction
{
    pass P0
    {
        //transforms
        WorldTransform[0]   = (tW);
        ViewTransform       = (tV);
        ProjectionTransform = (tP);

        //texturing
        Sampler[0] = (SrcSamp);
        TextureTransform[0] = (tTex);
        TexCoordIndex[0] = 0;
        TextureTransformFlags[0] = COUNT2;
        //Wrap0 = U;  // useful when mesh is round like a sphere
        
        Lighting       = FALSE;

        //shaders
        VertexShader = NULL;
        PixelShader  = NULL;
    }
}
